/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <glib.h>
#include <sys/time.h>
#include <pthread.h>
#include "cam.h"
#include "cam_error.h"
#include "cam_debug.h"
#include "cam_mm.h"
#include "cam_face_detection_focus.h"

static pthread_mutex_t face_detecton_mutex = PTHREAD_MUTEX_INITIALIZER;
static gboolean is_face_zoom = FALSE; /*note: the flag: now is in face zoom state or not*/
static CamFaceInfo face_info;


#define RECT_CORRECTION_VALUE (20)


/*note:check point whether in rect, @correction_value  for enhance/weaken the condition*/
static gboolean __check_point_whether_in_rect(int point_x,
							int point_y,
							CamVideoRectangle rect,
							int correction_value)
{

	if (point_x > (rect.x - correction_value)
		&& point_y > (rect.y - correction_value)
		&& point_x < (rect.w + rect.x + correction_value)
		&& point_y < (rect.h + rect.y + correction_value)) {
		DEBUG_TRACE("point is in rect");
		return TRUE;
	}
	return FALSE;
}

void cam_face_detection_focus_start_stream(camera_preview_data_s *preview_frame, void *user_data)
{

	cam_retm_if(user_data == NULL, "data is null");
	cam_retm_if(preview_frame == NULL, "data is null");
	struct appdata *ad = (struct appdata *)user_data;
	CamAppData *camapp = ad->camapp_handle;
	cam_retm_if(camapp == NULL, "camapp is null");

	if (cam_app_is_skip_video_stream()) {
		return;
	}
	if (cam_utils_set_guide_rect_color(ad) ==FALSE) {
		DEBUG_TRACE("cam_utils_set_guide_rect_color failed");
	}
	void *frame = NULL;
	switch (preview_frame->num_of_planes) {
	case 1:
		frame = preview_frame->data.single_plane.yuv;
		break;
	case 2:
		/*TODO: now do not support two planes*/
		return;
	case 3:
		/*TODO: now do not support three planes*/
		return;
	default:
		return;
	}
	pthread_mutex_lock(&face_detecton_mutex);
	int i;
	for(i = 0; i < face_info.count; i++ ) {

		if(i == 0){
			camapp->guide_rect_color = camapp->guide_rect_green;
		}else{
			camapp->guide_rect_color = camapp->guide_rect_white;
		}

		cam_utils_draw_guide_rectangle(ad,
						frame,
						face_info.faces[i].x,
						face_info.faces[i].y,
						face_info.faces[i].width,
						face_info.faces[i].height);

	}
	pthread_mutex_unlock(&face_detecton_mutex);
	return ;

}

gboolean cam_face_detection_focus_init()
{
	int err = 0;
	err = pthread_mutex_init(&(face_detecton_mutex), NULL);
	if (err != 0) {
		cam_critical(LOG_CAM, "Create face detection mutex failed");
		return FALSE;
	}
 	return TRUE;
}

gboolean cam_face_detection_focus_finish()
{
	pthread_mutex_destroy(&face_detecton_mutex);
	return TRUE;
}


/*note: in face detection focus mode, when face deteced , callback will be called*/
void cam_face_detection_focus_face_detected_cb(camera_detected_face_s *faces, int count, void *user_data)
{
	pthread_mutex_lock(&face_detecton_mutex);

	if(count > MAX_FACE_COUNT)
		count = MAX_FACE_COUNT;

	int i ;
	for(i = 0; i < count; i++ ) {

		face_info.faces[i].height = faces->height;
		face_info.faces[i].id = faces->id;
		face_info.faces[i].score = faces->score;
		face_info.faces[i].width = faces->width;
		face_info.faces[i].x = faces->x;
		face_info.faces[i].y = faces->y;
		faces++;
	}
	face_info.count = count;
	pthread_mutex_unlock(&face_detecton_mutex);

	return;
}

void cam_common_set_mouse_double_click_xy(Evas_Coord x, Evas_Coord y)
{
	struct appdata *ad = (struct appdata *)cam_appdata_get();
	cam_retm_if(ad == NULL, "appdata is NULL");

	CamAppData *camapp = NULL;
	camapp = ad->camapp_handle;
	cam_retm_if(camapp == NULL, "camapp is NULL");

	if (is_face_zoom) {

		cam_critical(LOG_CAM, "cancel face zoom");
		if (cam_mm_camera_cancel_face_zoom())
		is_face_zoom = FALSE;
		return;
	}

	pthread_mutex_lock(&face_detecton_mutex);

	if(face_info.count < 1 )
	{
		cam_debug(LOG_CAM, "not detected face");
		pthread_mutex_unlock(&face_detecton_mutex);
		return;
	}

	cam_debug(LOG_CAM, "x = %d, y = %d", x, y);

	/*calculate*/
	int i;
	for(i=0; i<face_info.count; i++)
	{
		CamVideoRectangle src;
		src.x = face_info.faces[i].x;
		src.y = face_info.faces[i].y;
		src.w = face_info.faces[i].width;
		src.h = face_info.faces[i].height;

		CamVideoRectangle dest;
		cam_utils_set_videos_xy_to_windows_xy(src, &dest,(void *)ad);

		cam_debug(LOG_CAM, "_x = %d, _y = %d", face_info.faces[i].x, face_info.faces[i].y);
		cam_debug(LOG_CAM, "cam_face_x = %d, cam_face_y = %d", dest.x, dest.y);

		gboolean point_in_rect = FALSE;
		point_in_rect = __check_point_whether_in_rect(x, y, dest, RECT_CORRECTION_VALUE);

		if (point_in_rect) {
			cam_debug(LOG_CAM, "in rect");
			if (is_face_zoom || (camapp->zoom_mode != camapp->zoom_min)) {
				if (cam_mm_camera_cancel_face_zoom()){
					is_face_zoom = FALSE;
					camapp->zoom_mode =  ZOOM_DEFAULT;
				}
			} else {
				if (cam_mm_set_camera_face_zoom(face_info.faces[i].id)) {
					is_face_zoom = TRUE;
				}
			}
		}else{
			cam_debug(LOG_CAM, "not matcing");
		}
	}
	pthread_mutex_unlock(&face_detecton_mutex);


}
void cam_face_detection_reset()
{
	face_info.count = 0;
}

//end file
