/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <stdio.h>
#include <string.h>
#include "cam_file.h"
#include "cam_error.h"
#include "cam_debug.h"
#include "cam_ta.h"

#include <sys/time.h>
#include <sys/times.h>
#include <sys/stat.h>
#include <dirent.h>
#include <media_content.h>

#define CAM_FILE_CONDITION_LENGTH 256

gboolean cam_file_init(GError **error)
{
	debug_fenter(LOG_UI);

	return TRUE;
}

void cam_file_finalize()
{
	debug_fenter(LOG_UI);
	return;
}

gboolean cam_file_get_size(const char *filename, guint64 *size)
{
	struct stat buf;

	if (stat(filename, &buf) != 0)
		return FALSE;
	*size = (guint64)buf.st_size;
	return TRUE;
}

gboolean cam_file_check_exists(gchar *filename)
{

	FILE *fd = fopen(filename, "r");
	if (fd == NULL) {
		return FALSE;
	} else {
		fclose(fd);
		return TRUE;
	}
}

gboolean cam_file_register(const gchar *filename, GError **error)
{
	int err_code = 0;

	CAM_TA_ACUM_ITEM_BEGIN("      cam_file_register", 0);

	cam_debug(LOG_UI, "");

	if (filename == NULL) {
		cam_critical(LOG_FILE, "The filename is null");
		CAM_TA_ACUM_ITEM_END("      cam_file_register", 0);
		return FALSE;
	}

	if (!g_file_test(filename, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_REGULAR)) {
		cam_critical(LOG_FILE, "The File is not existed");
		CAM_TA_ACUM_ITEM_END("      cam_file_register", 0);
		return FALSE;
	}
	if (g_str_has_suffix(filename, ".jpg")) {
		cam_debug(LOG_FILE, "jpg file!");

	} else if (g_str_has_suffix(filename, ".3gp")) {
		cam_debug(LOG_FILE, ".3gp file!");
	} else if (g_str_has_suffix(filename, ".mp4")) {
		cam_debug(LOG_FILE, ".mp4 file!");
	} else {
		cam_critical(LOG_FILE, "failed to get file type : [%s]",
			     filename);
		return FALSE;
	}

	media_info_h info = NULL;
	err_code = media_info_insert_to_db(filename, &info);

	if (err_code != MEDIA_CONTENT_ERROR_NONE) {
		cam_critical(LOG_FILE, "failed to media_file_register() : [%s], [%d]", filename, err_code);
		CAM_TA_ACUM_ITEM_END("      cam_file_register", 0);
		media_info_destroy(info);
		return FALSE;
	}
	media_info_destroy(info);

	cam_debug(LOG_FILE, "register success!");
	CAM_TA_ACUM_ITEM_END("      cam_file_register", 0);
	return TRUE;
}

gboolean cam_file_delete(const gchar *filename, GError **error)
{
	cam_debug(LOG_FILE, "%s", filename);

	if (!g_file_test(filename, G_FILE_TEST_IS_REGULAR)) {
		cam_warning(LOG_FILE, "invalid file:%s", filename);
		cam_set_error(error, CAM_ERROR_FILE_NOT_EXISTS,
			      "File not exists:%s", filename);
		return FALSE;
	}
	return (g_unlink(filename) == 0) ? TRUE : FALSE;
}

gboolean
cam_file_rename(const gchar *filename, const gchar *new_name, GError **error)
{
	gboolean bret = FALSE;

	cam_debug(LOG_FILE, "[%s] -> [%s]", filename, new_name);

	if (!g_file_test(filename, G_FILE_TEST_IS_REGULAR)) {
		cam_warning(LOG_FILE, "invalid file:%s", filename);
		cam_set_error(error, CAM_ERROR_FILE_NOT_EXISTS,
			      "File not exists:%s", filename);
		return FALSE;
	}

	bret = (g_rename(filename, new_name) == 0) ? TRUE : FALSE;

	if (!bret) {
		cam_critical(LOG_FILE, "operation failed");
	}

	if (!g_file_test(new_name, G_FILE_TEST_IS_REGULAR)) {
		cam_warning(LOG_FILE, "renamed file not exists:%s", new_name);
		bret = FALSE;
	}

	return bret;
}

const gchar *cam_file_get_internal_image_path(void)
{
	gchar * spath = INTERNAL_FILE_PATH;

	return spath;
}

const gchar *cam_file_get_internal_video_path(void)
{
	gchar * spath = INTERNAL_FILE_PATH;

	return spath;
}

const gchar *cam_file_get_external_image_path(void)
{
	gchar * spath = EXTERNAL_FILE_PATH;

	return spath;
}

const gchar *cam_file_get_external_video_path(void)
{
	gchar * spath = EXTERNAL_FILE_PATH;

	return spath;
}

gboolean cam_check_dir()
{
	DIR *internal_dir = NULL;
	DIR *external_dir = NULL;
	int ret;
	internal_dir = opendir(INTERNAL_FILE_PATH);

	if(internal_dir == NULL) {
		ret = mkdir(INTERNAL_FILE_PATH,0777);
		DEBUG_TRACE("ret = [%d]",ret);
		if(ret <0) {
			DEBUG_TRACE("Cannot make default path");
			/*closedir(internal_dir);*/
			return FALSE;
		}
	}
	if (internal_dir) {
		closedir(internal_dir);
		internal_dir = NULL;
	}

	external_dir = opendir(EXTERNAL_FILE_PATH);
	if(external_dir == NULL) {
		ret = mkdir(EXTERNAL_FILE_PATH,0777);
		DEBUG_TRACE("ret = [%d]",ret);
		if(ret <0) {
			DEBUG_TRACE("MMC is fault or not inserted");
		}

	}
	if (external_dir) {
		closedir(external_dir);
		external_dir = NULL;
	}

	return TRUE;

}
gchar *cam_file_get_next_filename_for_multishot(const gchar *storage_root,
						const gchar *first_file_name,
						const gint shot_num,
						const gchar *suffix)
{
	char *prefix = NULL;
	char *result = NULL;
	char *check_full_path = NULL;
	char *check_full_name = NULL;
	int cnt = 0;
	gboolean bFinal = FALSE;

	prefix = g_strdup_printf("%s/%s", storage_root, first_file_name);

	if (shot_num == 0) {
		check_full_name = g_strdup_printf("%s", prefix);
	} else {
		check_full_name =
		    g_strdup_printf("%s_%d", prefix, shot_num);
	}
	check_full_path = g_strdup_printf("%s%s", check_full_name, suffix);
	/*NOTE: if worst sceanio: always failed.set max count 1000*/
	#define MAX_SUB_NAME_NUMBER (1000)
	while (bFinal == FALSE && check_full_path && (cnt < MAX_SUB_NAME_NUMBER)) {
		if (!cam_file_check_exists(check_full_path)) {
			bFinal = TRUE;
			result = g_strdup(check_full_path);
		} else {
			cnt++;
			if (check_full_path)
				g_free(check_full_path);
			check_full_path = NULL;
			check_full_path =
			    g_strdup_printf("%s-%d%s", check_full_name, cnt, suffix);

			bFinal = FALSE;
		}
	}

	if (check_full_path)
		g_free(check_full_path);
	check_full_path = NULL;
	if (check_full_name)
		g_free(check_full_name);
	check_full_name = NULL;

	if (prefix)
		g_free(prefix);
	prefix = NULL;

	cam_debug(LOG_FILE, "%s", result);

	return result;
}

/*callback for:media_folder_foreach_media_from_db*/
static bool __cam_file_get_latest_file_path_cb(media_info_h item, void *user_data)
{
	char **file_url = (char **) user_data;
	if (media_info_get_file_path(item, file_url) != MEDIA_CONTENT_ERROR_NONE) {
		DEBUG_TRACE("Clone folder handle error");
		return FALSE;
	}

	return TRUE;
}

/*callback for:media_info_foreach_media_from_db*/
/*note:here have to use bool, not gboolean, for callback define*/
static bool __cam_file_get_cam_file_thumbnail_path_cb(media_info_h item, void *user_data)
{
	char **thumbnail_path = (char **) user_data;
	if (media_info_get_thumbnail_path(item, thumbnail_path) != MEDIA_CONTENT_ERROR_NONE) {
		DEBUG_TRACE("get thumbnail path error");
		return FALSE;
	}
	if (*thumbnail_path)
		DEBUG_TRACE("thumbnail_path = %s", *thumbnail_path);
	return TRUE;
}

/*callback for:media_folder_foreach_folder_from_db*/
static bool __cam_file_get_storage_folder_id_cb(media_folder_h item, void *user_data)
{
	char **folder_uuid = (char **) user_data;
	if (media_folder_get_folder_id(item, folder_uuid) != MEDIA_CONTENT_ERROR_NONE) {
		DEBUG_TRACE("Get folder id error");
		return FALSE;
	}

	return TRUE;
}

char *cam_file_get_cam_storage_folder_id(const gchar *storage_root)
{
	char condition[CAM_FILE_CONDITION_LENGTH + 1] = { '\0', };
	snprintf(condition, CAM_FILE_CONDITION_LENGTH, "%s = \'%s\'", FOLDER_PATH, (char *)storage_root);
	filter_h filter = NULL;
	int ret = media_filter_create(&filter);
	if(ret != MEDIA_CONTENT_ERROR_NONE) {
		DEBUG_TRACE("Fail to create filter");
		return NULL;
	}

	ret = media_filter_set_condition(filter, condition,	MEDIA_CONTENT_COLLATE_DEFAULT);
	if (ret != MEDIA_CONTENT_ERROR_NONE) {
		goto ERROR_THROW;
	}

	char *folder_uuid = NULL;
	ret = media_folder_foreach_folder_from_db(filter,
					  	__cam_file_get_storage_folder_id_cb,
					  	&folder_uuid);
	if (ret != MEDIA_CONTENT_ERROR_NONE) {
		goto ERROR_THROW;
	}

	if (folder_uuid) {
		if (filter)
			media_filter_destroy(filter);
		filter = NULL;
		return folder_uuid;
	} else {
		goto ERROR_THROW;
	}

ERROR_THROW:
	if (filter)
		media_filter_destroy(filter);
	filter = NULL;
	return NULL;
}

gboolean cam_file_get_cam_file_thumbnail_path(const gchar *file_path, gchar **thumbnail_path)
{
	if (thumbnail_path == NULL)
		return FALSE;
	char condition[CAM_FILE_CONDITION_LENGTH + 1] = { '\0', };
	snprintf(condition, CAM_FILE_CONDITION_LENGTH, "%s = \'%s\'", MEDIA_PATH, (char *)file_path);
	filter_h filter = NULL;
	int ret = media_filter_create(&filter);
	if(ret != MEDIA_CONTENT_ERROR_NONE) {
		DEBUG_TRACE("Fail to create filter");
		goto ERROR_THROW;
	}

	ret = media_filter_set_condition(filter, condition, MEDIA_CONTENT_COLLATE_DEFAULT);
	if (ret != MEDIA_CONTENT_ERROR_NONE) {
		goto ERROR_THROW;
	}

	char *temp_thumbnail_path = NULL;
	ret = media_info_foreach_media_from_db(filter,
					  	__cam_file_get_cam_file_thumbnail_path_cb,
					  	&temp_thumbnail_path);
	if (ret != MEDIA_CONTENT_ERROR_NONE) {
		goto ERROR_THROW;
	}
	*thumbnail_path = temp_thumbnail_path;
	if (filter)
		media_filter_destroy(filter);
	filter = NULL;
	return TRUE;

ERROR_THROW:
	if (filter)
		media_filter_destroy(filter);
	filter = NULL;
	return FALSE;
}

gchar *cam_file_get_last_file_path(void *data, const gchar *storage_root)
{
	struct appdata *ad =data;
	cam_retvm_if(ad == NULL, NULL, "appdata is NULL");
	cam_retvm_if(storage_root == NULL, NULL, "storage_root is NULL");
	DEBUG_TRACE("storage_root : %s", storage_root);


	/*get folder id*/

	char *folder_uuid = cam_file_get_cam_storage_folder_id(storage_root);
	if (!folder_uuid) {
		DEBUG_TRACE("not find %s", storage_root);
		return NULL;
	}

	char condition[CAM_FILE_PATH_MAX + 1] = { '\0', };

	snprintf(condition, CAM_FILE_PATH_MAX, "(%s=0 OR %s=1)", MEDIA_TYPE, MEDIA_TYPE);

	filter_h filter = NULL;
	int ret = media_filter_create(&filter);
	if(ret != MEDIA_CONTENT_ERROR_NONE) {
		DEBUG_TRACE("Fail to create filter");
		goto ERROR_THROW;
	}

	ret = media_filter_set_condition(filter, condition,	MEDIA_CONTENT_COLLATE_DEFAULT);
	if (ret != MEDIA_CONTENT_ERROR_NONE) {
		goto ERROR_THROW;
	}

	ret = media_filter_set_order(filter,
				     MEDIA_CONTENT_ORDER_DESC,
				     MEDIA_MODIFIED_TIME,
				     MEDIA_CONTENT_COLLATE_DEFAULT);
	if (ret != MEDIA_CONTENT_ERROR_NONE) {
		goto ERROR_THROW;
	}
	ret = media_filter_set_offset(filter,
					0,/*offset*/
					1);/*count*/
	char *file_url = NULL;
	ret = media_folder_foreach_media_from_db(folder_uuid,
						filter,
				       		__cam_file_get_latest_file_path_cb,
				       		&file_url);
	if (ret < 0) {
		DEBUG_TRACE("Failed to media_folder_foreach_media_from_db");
		goto ERROR_THROW;
	}

	if (file_url == NULL) {
		DEBUG_TRACE(" Failed ");
		goto ERROR_THROW;
	}
	/*make last filename */
	gchar last_file_path[255] = { 0, };
	snprintf(last_file_path, sizeof(last_file_path), "%s", file_url);
	free(file_url);
	file_url = NULL;
	media_filter_destroy(filter);
	filter = NULL;

	return g_strdup(last_file_path);

ERROR_THROW:
	if (filter)
		media_filter_destroy(filter);
	filter = NULL;
	return NULL;

}

gchar *cam_file_get_next_filename(const gchar *storage_root,
				  const gchar *prefix_string,
				  const gchar *suffix)
{
	gchar target[255] = { 0, };
	time_t t;
	struct tm tm;

	t = time(NULL);
	tzset();
		/*localtime_r : available since libc 5.2.5 */
	if(localtime_r(&t,&tm) == NULL) {
		return NULL;
	}
	snprintf(target, sizeof(target), "%s/%04i%02i%02i-%02i%02i%02i%s",
		 storage_root,
		 tm.tm_year + 1900, tm.tm_mon + 1, tm.tm_mday,
		 tm.tm_hour, tm.tm_min, tm.tm_sec, suffix);

	cam_debug(LOG_FILE, "filename: %s", target);

	return g_strdup(target);
}
