/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#include <X11/Xlib.h>
#include <X11/keysymdef.h>
#include <sys/wait.h>
#include <pthread.h>
#include <power.h>

#include "cam.h"
#include "cam_app.h"
#include "cam_sound.h"
#include "cam_file.h"
#include "cam_ta.h"
#include "cam_error.h"
#include "cam_mm.h"
#include "cam_rec.h"
#include "cam_indicator_edc_callback.h"
#include "cam_recording_edc_callback.h"
#include "camera_utils.h"
#include "cam_popup.h"
#include "cam_lbs.h"
#include "cam_common_edc_callback.h"
#include "cam_toolbar_edc_callback.h"

#ifdef USE_CAMERA_APP_SHUTTER_SOUND
static gint __cam_rec_sound_index_get(gint setting_index, gboolean start)
{
	gint index = -1;

	if (start) {		/*  start sound */
		switch (setting_index) {
		case CAM_SETTINGS_SOUND_1:
			index = CAM_SOUND_EFFECT_REC_START3;
			break;
		case CAM_SETTINGS_SOUND_2:
			index = CAM_SOUND_EFFECT_REC_START2;
			break;
		case CAM_SETTINGS_SOUND_3:
			index = CAM_SOUND_EFFECT_REC_START3;
			break;
		default:
			break;

		}
	} else {		/*  commit sound */
		switch (setting_index) {

		case CAM_SETTINGS_SOUND_1:
			index = CAM_SOUND_EFFECT_REC_COMMIT3;
			break;
		case CAM_SETTINGS_SOUND_2:
			index = CAM_SOUND_EFFECT_REC_COMMIT2;
			break;
		case CAM_SETTINGS_SOUND_3:
			index = CAM_SOUND_EFFECT_REC_COMMIT3;
			break;
		default:
			break;
		}
	}
	return index;
}
#endif
static gchar *__get_tmpfile_name()
{
	char tmpfile_name[256] = { 0 };
	gchar *file_path = NULL;
	file_path = (gchar *)cam_app_get_target_path();

	snprintf(tmpfile_name, sizeof(tmpfile_name), "%s%s", file_path,
		 TMPFILE_PREFIX);
	DEBUG_TRACE("#####################tmpfile_name=%s", tmpfile_name);

	if (g_file_test(tmpfile_name, G_FILE_TEST_EXISTS)) {
		DEBUG_TRACE("Error : tmp file exist on the %s", file_path);
		cam_remove_tmp_file();
	}

	return g_strdup(tmpfile_name);
}

gboolean cam_video_record_start(void *data)
{
	struct appdata *ad = (struct appdata *)data;
	CamAppData *camapp = NULL;
	gchar *filename = NULL;

	gint64 remain = 0;

	debug_fenter(LOG_UI);

	cam_retvm_if(ad == NULL, FALSE, "appdata is NULL");
	camapp = ad->camapp_handle;
	cam_retvm_if(camapp == NULL, FALSE, "camapp_handle is NULL");

	if (ad->is_recording == TRUE) {
		DEBUG_TRACE("Camera app is recording");
		return TRUE;
	} else {
		cam_critical(LOG_UI, "start reocrd");
	}

	if (ad->is_rotating == TRUE) {
		DEBUG_TRACE("now in rotating target, do not start recording");
		return TRUE;
	}

	DEBUG_TRACE("start recording !!");

	if (ad->timer_activated) {
		CAM_UI_LOG("timer activated!");
		return true;
	}
	gchar* target_path = (gchar*)cam_app_get_target_path();
	if (target_path != NULL) {
		remain = cam_get_free_space(target_path);
	}

	if (remain <= REC_MMS_MIN_SPACE) {
		cam_app_notice_popup(ad, dgettext(PACKAGE, "IDS_CAM_BODY_UNABLE_TO_SAVE_NOT_ENOUGH_MEMORY"),
				     cam_app_popup_response_cb);
		return TRUE;
	}

	if (camapp->gps) {
		int gps_level = CAM_LBS_STATE_DISABLE;
		gps_level = cam_lbs_get_state();
		if (gps_level >=CAM_LBS_STATE_SERVICE_ENABLE) {
			double longitude = -1.0;
			double latitude = -1.0;
			double altitude = -1.0;
			time_t time_stamp = -1.0;
			if (cam_lbs_get_current_position(&longitude, &latitude, &altitude, &time_stamp)) {
				DEBUG_TRACE("GEO TAG [longitude = %d, latitude = %d, altitude = %d]", longitude, latitude, altitude);
				if ( !cam_mm_set_gps_data(latitude, longitude, (double)altitude))
					DEBUG_TRACE("cam_mm_set_gps_data failed");
			}
		}
	}

	filename = __get_tmpfile_name();

	if (filename) {
		cam_debug(LOG_UI, "FILE NAME : %s", filename);
		if (!cam_mm_set_filename(filename)) {
			cam_critical(LOG_MM, "cannot set filename");
			return FALSE;
		}
		free(filename);
		filename = NULL;
	} else {
		cam_critical(LOG_MM, "cannot get filename");
		return FALSE;
	}

	/* flash on */
	if (camapp->flash == CAM_FLASH_ON) {
		if (!cam_mm_set_flash(CAM_FLASH_MOVIE_ON)) {
			cam_critical(LOG_MM, "cam_mm_set_flash failed");
		}
	} else {
		if (!cam_mm_set_flash(CAM_FLASH_OFF)) {
			cam_critical(LOG_MM, "cam_mm_set_flash failed");
		}
	}

#ifdef USE_CAMERA_APP_SHUTTER_SOUND
/* play start effect sound */
	cam_sound_play(__cam_rec_sound_index_get(camapp->shutter_sound, TRUE),
		       ad);
#endif
/* set sound path for recording MIC */
	cam_sound_set_mic();

/* set orient tag */
	cam_set_orient_value(ad);

/* mm rec */
	if (!cam_mm_rec_start(NULL)) {
		cam_critical(LOG_MM, "cam_mm_rec_start failed");
		cam_app_notice_popup(data,
					dgettext(PACKAGE, "IDS_CAM_POP_ERROR_RECORDING_FAIL"),
					cam_app_timeout_notice_response_cb);
		return FALSE;
	}
	/* set recording state */
	ad->is_recording = TRUE;

/* change recording view */

	if (load_recording_edje(ad)) {
		cam_critical(LOG_UI, "load_recording_edje failed");
		cam_video_record_cancel(ad);
		return FALSE;
	}

	if (ad->indicator_edje)
		cam_indicator_update(ad);
	else
		cam_indicator_create(ad);

	settings_guideline_refresh(ad);
	return TRUE;
}

gboolean cam_video_record_resume(void *data)
{
	struct appdata *ad = (struct appdata *)data;
	cam_retvm_if(ad == NULL, FALSE, "appdata is NULL");

	if (!cam_mm_rec_start(NULL)) {
		cam_critical(LOG_MM, "cam_mm_rec_start failed");
		cam_app_notice_popup(data,
					dgettext(PACKAGE, "IDS_CAM_POP_ERROR_RECORDING_FAIL"),
					cam_app_timeout_notice_response_cb);
		return FALSE;
	}

	return TRUE;
}

gboolean cam_video_record_pause(void *data)
{
	struct appdata *ad = (struct appdata *)data;
	cam_retvm_if(ad == NULL, FALSE, "appdata is NULL");

	if (!cam_mm_rec_pause(NULL)) {
		cam_critical(LOG_MM, "cam_mm_rec_start failed");
		return FALSE;
	}
	return TRUE;
}
Eina_Bool cam_video_idler_record_stop(void *data)
{
	struct appdata *ad = (struct appdata *)data;
	cam_retvm_if(ad == NULL, FALSE, "appdata is NULL");

	cam_video_record_stop(ad);

	return ECORE_CALLBACK_CANCEL;
}

gboolean cam_video_record_stop(void *data)
{
	struct appdata *ad = (struct appdata *)data;
	CamAppData *camapp = NULL;
	gboolean to_stop = FALSE;

	debug_fenter(LOG_UI);

	cam_retvm_if(ad == NULL, FALSE, "appdata is NULL");
	camapp = ad->camapp_handle;
	cam_retvm_if(camapp == NULL, FALSE, "camapp_handle is NULL");

	if (ad->recording_commit) {
		ecore_idler_del(ad->recording_commit);
		ad->recording_commit = NULL;
	}
/*  check to_stop val */

	if (!cam_mm_stop_focusing()){
		cam_critical(LOG_MM, "cam_mm_stop_focusing error ");
	}
/* mm stop rec */
	if (!cam_mm_rec_stop(to_stop)) {
		cam_critical(LOG_MM, "cam_mm_rec_stop failed");
		if (!cam_mm_rec_cancel())
			cam_critical(LOG_MM, "cam_mm_rec_cancel failed");

		cam_app_notice_popup(data,
					dgettext(PACKAGE, "IDS_CAM_POP_ERROR_RECORDING_FAIL"),
					cam_app_timeout_notice_response_cb);
	}

/* flash off */
	if (!cam_mm_set_flash(camapp->flash)) {
		cam_critical(LOG_MM, "cam_mm_set_flash failed");
	}

/* set sound path for effect sound */
#ifdef USE_CAMERA_APP_SHUTTER_SOUND
/* play end effect sound */
	cam_sound_play(__cam_rec_sound_index_get(camapp->shutter_sound, FALSE), ad);
#endif
/*      cam_sound_play(CAM_SOUND_EFFECT_REC_COMMIT); */

	return TRUE;
}

gboolean cam_video_record_cancel(void *data)
{
	struct appdata *ad = (struct appdata *)data;
	CamAppData *camapp = NULL;

	debug_fenter(LOG_UI);

	cam_retvm_if(ad == NULL, FALSE, "appdata is NULL");
	camapp = ad->camapp_handle;
	cam_retvm_if(camapp == NULL, FALSE, "camapp_handle is NULL");

/*mm stop rec */
	if (!cam_mm_rec_cancel(NULL)) {
		cam_critical(LOG_MM, "cam_mm_rec_cancel failed");
	}

	/* change to camera mode*/
		/*
		*	TODO: if for any reason, not run here, there will be caused now now recording, but in record mode
		*	So: I suggest while click capture button: judge now state and whether recording.
		*/

	if(ad->launching_mode == CAM_LAUNCHING_MODE_NORMAL) {
		if (ad->ug_state != CAM_UG_PAUSE_STATE) {/*if now is pause state, so do not set mode, just set while resume*/

			GValue value = {0, };
			CAM_GVALUE_SET_INT(value, CAM_CAMERA_MODE);
			if (!cam_handle_value_set(ad, PROP_MODE, &value)) {
				return FALSE;
			}

			cam_app_update_quickview_icon(ad);
		}
	}

	unload_recording_edje(ad);
	camapp->rec_elapsed = 0;
	cam_toolbar_update(ad);

	if (ad->indicator_edje)
		cam_indicator_update(ad);
	else
		cam_indicator_create(ad);

	settings_guideline_refresh(ad);
	cam_remove_tmp_file();
	return TRUE;
}

/* Idler of capture callback management. wh01.cho@samsung.com. 2010-12-15. */
Eina_Bool cam_video_capture_handle_idler(void *itm)
{
	CamIdlerItem *item = (CamIdlerItem *) itm;
	struct appdata *ad = NULL;
	cam_retvm_if(!item, ECORE_CALLBACK_CANCEL, "CamIdlerItem is NULL");

	ad = (struct appdata *)item->data;
	cam_retvm_if(!ad, ECORE_CALLBACK_CANCEL, "appdata is NULL");

	ad->capture_cb_list = g_list_remove(ad->capture_cb_list, item->idler);
	free(item);
	item = NULL;

	return cam_video_capture_handle(ad);
}

gboolean cam_rec_save_and_register_video_file(void *data)
{
	struct appdata *ad = (struct appdata *)data;
	CamAppData *camapp = NULL;
	cam_retvm_if(ad == NULL, FALSE, "appdata is NULL");
	camapp = ad->camapp_handle;
	cam_retvm_if(camapp == NULL, FALSE, "camapp_handle is NULL");
	cam_retv_if(camapp->filename == NULL, FALSE);

	GError *error = NULL;
	cam_debug(LOG_FILE, "tmp file : %s", camapp->filename);

	gchar *filename = NULL;
	filename = cam_app_get_next_filename(CAM_FILE_EXTENTION_VIDEO);

	if (filename) {
		cam_debug(LOG_UI, "FILE NAME : %s", filename);

		int ret = 0;

		ret = rename(camapp->filename, filename);

		if( ret != 0)
			cam_critical(LOG_FILE, "rename is error %d", ret );

		sync();
		free(camapp->filename);
		camapp->filename = strdup(filename);

		free(filename);
		filename = NULL;

		if (!g_file_test(camapp->filename, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_REGULAR)) {
			cam_critical(LOG_FILE, "The File is not existed %s",camapp->filename );
			return ECORE_CALLBACK_CANCEL;
		}
	} else {
		cam_critical(LOG_MM, "cannot get filename");
		return ECORE_CALLBACK_CANCEL;
	}
	if (!cam_file_register(camapp->filename, &error)) {
		cam_critical(LOG_FILE, "cam_file_register fail");
		if (error != NULL) {
			cam_critical(LOG_FILE,
				     "cam_file_register error [%s]",
				     error->message);
			g_error_free(error);
			error = NULL;
		}
	}
	if (ad->launching_mode == CAM_LAUNCHING_MODE_NORMAL)
		cam_app_update_thumbnail();

	/*TODO: use pipe*/
	if (camapp->review) {
		DEBUG_TRACE(" review on in recording mode");
		cam_utils_request_main_pipe_handler(ad, NULL, CAM_MAIN_PIPE_OP_TYPE_RUN_IMAGE_VIEWER);
	}
	return TRUE;
}

Eina_Bool cam_video_capture_handle(void *data)
{
	struct appdata *ad = (struct appdata *)data;
	CamAppData *camapp = NULL;

		/* set recording state */
	ad->is_recording = FALSE;

	cam_retvm_if(ad == NULL, ECORE_CALLBACK_CANCEL, "appdata is NULL");
	camapp = ad->camapp_handle;
	cam_retvm_if(camapp == NULL, ECORE_CALLBACK_CANCEL,
		     "camapp_handle is NULL");

	cam_debug(LOG_FILE, "");

	camapp->rec_elapsed = 0;
	SHOW_EVAS_OBJECT(ad->indicator_edje);
	unload_recording_edje(ad);
	camapp->rec_elapsed = 0;
	cam_toolbar_update(ad);
	if (ad->indicator_edje)
		cam_indicator_update(ad);
	else
		cam_indicator_create(ad);
	settings_guideline_refresh(ad);

	if (camapp->filename) {
		ad->remained_count--;
		indicator_update_remain_count(ad);
	}
	g_queue_push_tail(ad->file_reg_queue, strdup(REC_FILE_SAVE_REG));
	pthread_cond_signal(&ad->file_reg_cond);

	switch (camapp->rec_stop_type) {
	case CAM_REC_STOP_UNEXPECTED:
		break;
	case CAM_REC_STOP_NORMAL:
		break;
	case CAM_REC_STOP_LOW_BATTERY:
		cam_app_exit(ad);
		return ECORE_CALLBACK_CANCEL;
		break;
	case CAM_REC_STOP_ASM:
		break;
	case CAM_REC_STOP_MAX_SIZE:
		break;
	case CAM_REC_STOP_TIME_LIMIT:
		break;
	case CAM_REC_STOP_NO_SPACE:
		cam_app_notice_popup(ad, dgettext(PACKAGE, "IDS_CAM_POP_VIDEOMEMORYFULL"),
				     cam_app_popup_response_cb);
		break;
	case CAM_REC_STOP_USB_CONN:
		break;
	case CAM_REC_STOP_POWER_KEY:
		power_unlock_state(POWER_STATE_NORMAL);
		break;
	default:
		break;
	}
	camapp->rec_stop_type = CAM_REC_STOP_UNEXPECTED;

	return ECORE_CALLBACK_CANCEL;
}
