/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#include <sound_manager.h>
#include <wav_player.h>
#include <pthread.h>
#include <vconf.h>
#include <errno.h>
#include <sys/time.h>
#include "cam.h"
#include "cam_sound.h"
/*
#include "cam_sound_private.h"
*/
#include "cam_debug.h"
#include "cam_ta.h"
#include "cam_mm.h"

#define SOUND_TIMEOUT_SEC 	2

static gchar **sound_files = NULL;
#ifdef USE_CAMERA_APP_SHUTTER_SOUND
pthread_mutex_t sound_lock = PTHREAD_MUTEX_INITIALIZER;
pthread_cond_t sound_cond = PTHREAD_COND_INITIALIZER;
#endif

static void cam_sound_callback(int id, void *data);

gboolean cam_sound_init(GError **error)
{
	debug_fenter(LOG_UI);

	if (!sound_files) {
		sound_files = g_new0(gchar *, CAM_SOUND_EFFECT_NUM);
		g_return_val_if_fail(sound_files, FALSE);

		int i;
		for (i = 0; i < CAM_SOUND_EFFECT_NUM; i++)
			sound_files[i] = NULL;
#ifdef USE_CAMERA_APP_SHUTTER_SOUND
		cam_sound_set(CAM_SOUND_EFFECT_SHUTTER1, SOUND_PATH "/Shutter_01.wav");
		cam_sound_set(CAM_SOUND_EFFECT_SHUTTER2, SOUND_PATH "/Shutter_02.wav");
		cam_sound_set(CAM_SOUND_EFFECT_SHUTTER3, SOUND_PATH "/Shutter_03.wav");
		cam_sound_set(CAM_SOUND_EFFECT_REC_START1, SOUND_PATH "/cam_start_01.wav");
		cam_sound_set(CAM_SOUND_EFFECT_REC_START2, SOUND_PATH "/cam_start_02.wav");
		cam_sound_set(CAM_SOUND_EFFECT_REC_START3, SOUND_PATH "/cam_start_03.wav");
		cam_sound_set(CAM_SOUND_EFFECT_REC_COMMIT1, SOUND_PATH "/cam_stop_01.wav");
		cam_sound_set(CAM_SOUND_EFFECT_REC_COMMIT2, SOUND_PATH "/cam_stop_02.wav");
		cam_sound_set(CAM_SOUND_EFFECT_REC_COMMIT3, SOUND_PATH "/cam_stop_03.wav");
#endif
		cam_sound_set(CAM_SOUND_EFFECT_TICK, SOUND_PATH "/count.wav");
		cam_sound_set(CAM_SOUND_EFFECT_AF_OK, SOUND_PATH "/af_ok.wav");
		cam_sound_set(CAM_SOUND_EFFECT_AF_FAIL, SOUND_PATH "/af_fail.wav");

	}
#ifdef USE_CAMERA_APP_SHUTTER_SOUND
	pthread_mutex_init(&sound_lock, NULL);
	pthread_cond_init(&sound_cond, NULL);
#endif
	return TRUE;
}

void cam_sound_finalize(void)
{
	debug_fenter(LOG_UI);

	if (sound_files) {
		int i;
		for (i = 0; i < CAM_SOUND_EFFECT_NUM; i++) {
			if (sound_files[i]) {
				g_free(sound_files[i]);
				sound_files[i] = NULL;
			}
		}
		g_free(sound_files);
		sound_files = NULL;
	}
#ifdef USE_CAMERA_APP_SHUTTER_SOUND
	pthread_mutex_destroy(&sound_lock);
	pthread_cond_destroy(&sound_cond);
#endif
}

void cam_sound_set(int index, const gchar *file)
{
	debug_msg(LOG_UI, "%d:%s", index, file ? file : "NULL");
	g_return_if_fail(sound_files);
	if (sound_files[index]) {
		g_free(sound_files[index]);
		sound_files[index] = NULL;
	}
	if (file)
		sound_files[index] = g_strdup(file);
}

const gchar *cam_sound_get(int index)
{
	debug_fenter(LOG_UI);
	g_return_val_if_fail(sound_files, NULL);
	return sound_files[index];
}

gboolean cam_sound_is_silent_mode(void)
{
	int sound_on = -1;
	if (!vconf_get_bool(VCONFKEY_SETAPPL_SOUND_STATUS_BOOL, &sound_on)) {
		if (sound_on == FALSE) {
		    	cam_debug(LOG_SND, "sound off mode ");
			return TRUE;
		}
	}
	return FALSE;
}

gboolean cam_sound_set_mic()
{
	int ret = SOUND_MANAGER_ERROR_NONE;
	bool is_available = FALSE;
	sound_route_e route_to_active = SOUND_ROUTE_IN_MIC;

	/* Check if earphone mic is available */
	is_available = sound_manager_is_route_available(SOUND_ROUTE_IN_WIRED_ACCESSORY);

	/* If earphone mic is available, use earphone mic, otherwise use phone mic */
	route_to_active = (is_available)? SOUND_ROUTE_IN_WIRED_ACCESSORY : SOUND_ROUTE_IN_MIC;
	cam_debug(LOG_SND, "sound route is [%d]", route_to_active);

	ret = sound_manager_set_active_route(route_to_active);
	if (ret != SOUND_MANAGER_ERROR_NONE) {
		cam_critical(LOG_SND, "sound_manager_set_active_route [%d]) failed", route_to_active);
	}

	return ret;
}

static void cam_sound_callback(int id, void *data)
{
	struct appdata *ad = (struct appdata *)data;
	cam_ret_if(ad == NULL);

#ifdef USE_CAMERA_APP_SHUTTER_SOUND
	cam_debug(LOG_MM, "release cond");

	/*We can not set value to  sond_cond.
	   Bacause it can be assigned PTHREAD_COND_INITIALIZER only.
	   PTHREAD_COND_INITIALIZER is the MECRO of pthread.
	   So We use the sound_files, When the sound_cond is destroied, this value set to NULL.
	 */
	if (sound_files)
		pthread_cond_broadcast(&sound_cond);

	ad->play_rec_sound--;
	DEBUG_TRACE("ad->play_rec_sound=%d", ad->play_rec_sound);
#endif
	return;
}

gboolean cam_sound_play(int index, void *data)
{
	struct appdata *ad = (struct appdata *)data;
	cam_retv_if(ad == NULL, FALSE);

#ifdef CAMERA_MACHINE_I686
	return TRUE;
#endif
	int step = 0, handle = 0;
#ifdef USE_CAMERA_APP_SHUTTER_SOUND
	gboolean sync = FALSE;
	struct timespec timeout;
	struct timeval tv;
#endif

	g_return_val_if_fail(sound_files, FALSE);
	g_return_val_if_fail(index >= 0 && index < CAM_SOUND_EFFECT_NUM, FALSE);

	debug_msg(LOG_UI, "%s", sound_files[index]);

	if (!sound_files[index]) {
		return FALSE;
	}

	if (cam_sound_is_silent_mode()) {
		warn_msg(LOG_SND, "Silent mode now");
		return FALSE;
	}
#ifdef USE_CAMERA_APP_SHUTTER_SOUND
	if (index == CAM_SOUND_EFFECT_REC_START1
	    || index == CAM_SOUND_EFFECT_REC_START2
	    || index == CAM_SOUND_EFFECT_REC_START3 || FALSE) {
		sync = TRUE;
	}
#endif

	cam_debug(LOG_MM, " Volume step : %d", step);
#ifdef USE_CAMERA_APP_SHUTTER_SOUND

	if (EBUSY == pthread_mutex_trylock(&sound_lock)) {
		cam_critical(LOG_SND,
			     "another effect sound is playing, try later");
		return FALSE;
	}
#endif

	if (WAV_PLAYER_ERROR_NONE ==
	    wav_player_start(sound_files[index], SOUND_TYPE_MEDIA,  cam_sound_callback, ad, &handle)) {
#ifdef USE_CAMERA_APP_SHUTTER_SOUND
		ad->play_rec_sound++;
		DEBUG_TRACE("ad->play_rec_sound=%d", ad->play_rec_sound);
		if (sync) {
			gettimeofday(&tv, NULL);
			timeout.tv_sec = tv.tv_sec + SOUND_TIMEOUT_SEC;
			timeout.tv_nsec = tv.tv_usec;
			if (ETIMEDOUT ==
			    pthread_cond_timedwait(&sound_cond, &sound_lock,
						   &timeout)) {
				if (handle > 0) {
					if (WAV_PLAYER_ERROR_NONE != wav_player_stop(handle) {
						cam_critical(LOG_SND, "sound play failed");
					}
				}
			}
		}
#endif
	} else {
		cam_critical(LOG_SND, "effect sound play failed");
#ifdef USE_CAMERA_APP_SHUTTER_SOUND
		pthread_mutex_unlock(&sound_lock);
#endif
		return FALSE;
	}
#ifdef USE_CAMERA_APP_SHUTTER_SOUND
	pthread_mutex_unlock(&sound_lock);
#endif
	return TRUE;
}
