/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#include <stdio.h>
#include "camera_utils.h"

#include "edc_string.h"

#include "cam_common_edc_callback.h"
#include "cam_toolbar_edc_callback.h"
#include "cam_indicator_edc_callback.h"
#include "cam_zoom_edc_callback.h"
#include "cam_ev_edc_callback.h"
#include "cam_face_detection_focus.h"
#include "cam_ui_effect_utils.h"
#include "cam_menu_composer.h"
#include "cam_capacity_type.h"
#include "cam_device_capacity.h"
#include "cam_mm.h"


#define ZOOM_THRESHOLD	0.02

static double prev_zoom = 0;

static Evas_Event_Flags __pinch_start(void *data, void *event_info)
{
	struct appdata *ad = (struct appdata *)data;
	cam_retvm_if(ad == NULL, EVAS_EVENT_FLAG_NONE, "appdata is NULL");
	CamAppData *camapp = ad->camapp_handle;
	cam_retvm_if(camapp == NULL, EVAS_EVENT_FLAG_NONE, "camapp_handle is NULL");

	DEBUG_TRACE("__pinch_start");

	if (camapp->touch_af_state == CAM_TOUCH_AF_STATE_DOING) {
		DEBUG_TRACE("touch_af_state is [%d], do not zoom", camapp->touch_af_state);
		return EVAS_EVENT_FLAG_NONE;
	}

	DEL_EVAS_OBJECT(ad->focus_edje);
	cam_mm_stop_focusing();

	prev_zoom = 0;

	if (ad->toolbar_setting_popup || ad->setting_ctxpopup) {
		ad->displayed_setting_popup = DISPLAY_NONE_POPUP;
		cam_toolbar_setting_popup_destroy(ad);
	}

	if (ad->edit_shortcuts_popup) {
		toolbar_destroy_shortcuts_popup(ad);
	}

	if (ad->zoom_edje)
		unload_zoom_edje(ad);

	if (ad->ev_edje)
		unload_ev_edje(ad);

	cam_zoom_create_pinch_edje(ad);

	return EVAS_EVENT_FLAG_NONE;
}

static Evas_Event_Flags __pinch_end(void *data, void *event_info)
{
	struct appdata *ad = (struct appdata *)data;
	cam_retvm_if(ad == NULL, EVAS_EVENT_FLAG_NONE, "appdata is NULL");

	DEBUG_TRACE("__pinch_end");

	DEL_EVAS_OBJECT(ad->pinch_edje)
	cam_reset_focus_mode(ad);

	return EVAS_EVENT_FLAG_NONE;
}

static Evas_Event_Flags __pinch_move(void *data, void *event_info)
{
	Elm_Gesture_Zoom_Info *ev = (Elm_Gesture_Zoom_Info *)event_info;
	cam_retvm_if(ev == NULL, EVAS_EVENT_FLAG_NONE, "event_info is NULL");
	struct appdata *ad = (struct appdata *)data;
	cam_retvm_if(ad == NULL, EVAS_EVENT_FLAG_NONE, "appdata is NULL");
	CamAppData *camapp = ad->camapp_handle;
	cam_retvm_if(camapp == NULL, EVAS_EVENT_FLAG_NONE, "camapp_handle is NULL");

	if (camapp->touch_af_state == CAM_TOUCH_AF_STATE_DOING) {
		DEBUG_TRACE("touch_af_state is [%d], do not zoom", camapp->touch_af_state);
		return EVAS_EVENT_FLAG_NONE;
	}

	if (ad->pinch_edje == NULL)
		__pinch_start(ad, NULL);

	double diff = prev_zoom - ev->zoom;
	if (ABS(diff) < ZOOM_THRESHOLD) {
		return EVAS_EVENT_FLAG_NONE;
	}

	prev_zoom = ev->zoom;

	gboolean is_zoom_in = FALSE;
	if (diff > 0)
		is_zoom_in = FALSE;
	else
		is_zoom_in = TRUE;

	int state = 0;
	state = cam_mm_get_state();
	cam_debug(LOG_UI, " state = %d", state);
	if (camapp->camera_mode == CAM_CAMERA_MODE) {
		if (state == CAMERA_STATE_PREVIEW
			|| state == CAMERA_STATE_CAPTURED) {
			if (!camapp->zoom_lock) {
				cam_pinch_zoom_in(ad, is_zoom_in, 1);
				return EVAS_EVENT_FLAG_NONE;
			}
		}
	} else if (camapp->camera_mode == CAM_CAMCORDER_MODE) {
		if (state == RECORDER_STATE_READY
			|| state == RECORDER_STATE_RECORDING
			|| state == RECORDER_STATE_PAUSED) {
			if (!camapp->zoom_lock
					&& camapp->recording_mode != CAM_RECORD_SLOW) {
				cam_pinch_zoom_in(ad, is_zoom_in, 1);
				return EVAS_EVENT_FLAG_NONE;
			}
		}
	}

	return EVAS_EVENT_FLAG_NONE;
}

static void __mouse_click(void *data)
{
	struct appdata *ad = (struct appdata *)data;
	cam_retm_if(ad == NULL, "appdata is NULL");

	if (ad->zoom_edje)
		unload_zoom_edje(ad);

	if (ad->ev_edje)
		unload_ev_edje(ad);

	if (ad->setting_ctxpopup) {
		ad->displayed_setting_popup = DISPLAY_NONE_POPUP;
		cam_toolbar_setting_popup_destroy(ad);
	} else if (ad->toolbar_setting_popup) {
		cam_toolbar_setting_popup_destroy(ad);
	} else if (ad->edit_shortcuts_popup) {
		toolbar_destroy_shortcuts_popup(ad);
	}

	return;
}

static void __mouse_up_cb(void *data, Evas * evas, Evas_Object *obj, void *event_info)
{
	Evas_Event_Mouse_Up *ev = (Evas_Event_Mouse_Up *)event_info;
	cam_retm_if(ev == NULL, "event_info is NULL");
	struct appdata *ad = (struct appdata *)data;
	cam_retm_if(ad == NULL, "appdata is NULL");

	if (ad->is_rotating) {
		cam_critical(LOG_MM, "rotating...");
		return;
	}

	unsigned int focus_capacity = 0;
	GetCamDevFocusCaps(&focus_capacity, ad);

	if (ev->flags & EVAS_BUTTON_DOUBLE_CLICK) {
		DEBUG_TRACE("double click");
		return;
	}

	if (focus_capacity & CAM_CP_FOCUS_AUTO_FOCUS) {
		CAM_UI_LOG("mouse up :[%d, %d]", ev->canvas.x, ev->canvas.y);
		cam_mouse_button_up(ad, ev);
	}

	__mouse_click(ad);
	cam_app_timeout_checker_update();

	return;
}

static void __mouse_down_cb(void *data, Evas * evas, Evas_Object *obj, void *event_info)
{
	Evas_Event_Mouse_Down *ev = (Evas_Event_Mouse_Down *) event_info;
	cam_retm_if(ev == NULL, "event_info is NULL");
	struct appdata *ad = (struct appdata *)data;
	cam_retm_if(ad == NULL, "appdata is NULL");
	CamAppData *camapp = ad->camapp_handle;
	cam_retm_if(camapp == NULL, "camapp is NULL");

	unsigned int focus_capacity = 0;
	GetCamDevFocusCaps(&focus_capacity, ad);

	if (ad->is_rotating) {
		cam_critical(LOG_MM, "rotating...");
		return;
	}

	if (ad->is_processing) {
		cam_critical(LOG_MM, "bust shot popup is displaying...");
		return;
	}

	if (ev->flags & EVAS_BUTTON_DOUBLE_CLICK) {
		DEBUG_TRACE("double click");

		if(camapp->af_mode == CAM_FOCUS_FACE){
			cam_common_set_mouse_double_click_xy(ev->canvas.x, ev->canvas.y);
		}
		return;
	}

	if (focus_capacity & CAM_CP_FOCUS_AUTO_FOCUS) {
		CAM_UI_LOG("mouse down :[%d, %d]", ev->canvas.x, ev->canvas.y);
		cam_mouse_button_down(ad, ev);
	}

	cam_app_timeout_checker_update();

	return;
}

static void __mouse_move_cb(void *data, Evas * evas, Evas_Object *obj, void *event_info)
{
	cam_app_timeout_checker_update();
	return;
}

void define_mouse_callback(void *data)
{
	struct appdata *ad = (struct appdata *)data;
	cam_retm_if(ad == NULL, "appdata is NULL");

	unsigned int capacity = 0;
	GetCamDevFuncCaps(&capacity, ad);

	Evas_Object *eo = (Evas_Object *)edje_object_part_object_get(_EDJ(ad->ug_base), "mouse_event_layer");

	evas_object_event_callback_add(eo, EVAS_CALLBACK_MOUSE_DOWN, __mouse_down_cb, ad);
	evas_object_event_callback_add(eo, EVAS_CALLBACK_MOUSE_UP, __mouse_up_cb, ad);
	evas_object_event_callback_add(eo, EVAS_CALLBACK_MOUSE_MOVE, __mouse_move_cb, ad);

	if (capacity & CAM_CP_FUNC_ZOOM) {
		if(ad->gesture == NULL)
			ad->gesture = elm_gesture_layer_add(eo);

		elm_gesture_layer_cb_set(ad->gesture, ELM_GESTURE_ZOOM, ELM_GESTURE_STATE_START, __pinch_start, ad);
		elm_gesture_layer_cb_set(ad->gesture, ELM_GESTURE_ZOOM, ELM_GESTURE_STATE_MOVE, __pinch_move, ad);
		elm_gesture_layer_cb_set(ad->gesture, ELM_GESTURE_ZOOM, ELM_GESTURE_STATE_END, __pinch_end, ad);
		elm_gesture_layer_cb_set(ad->gesture, ELM_GESTURE_ZOOM, ELM_GESTURE_STATE_ABORT, __pinch_end, ad);
		elm_gesture_layer_attach(ad->gesture, eo);
	} else {
		DEL_EVAS_OBJECT(ad->gesture);
	}
}

void cam_layout_del_all(void *data)
{
	CAM_UI_LOG();
	struct appdata *ad = (struct appdata *)data;
	cam_toolbar_destroy(ad);

	if (ad->indicator_edje)
		cam_indicator_destory(ad);

	if (ad->ev_edje)
		unload_ev_edje(ad);

	if (ad->zoom_edje)
		unload_zoom_edje(ad);

	ad->displayed_setting_popup = DISPLAY_NONE_POPUP;
}

int cam_convert_shot_mode_to_menu_item(int mode)
{
	switch(mode)
	{
		case CAM_SINGLE_MODE:
			return CAM_MENU_SHOOTING_SINGLE;
		case CAM_SELF_SINGLE_MODE:
			return CAM_MENU_SHOOTING_SELF_SINGLE;
		default:
			return CAM_MENU_SHOOTING_SINGLE;
	}
}

int cam_convert_flash_to_menu_item(int mode)
{
	switch(mode)
	{
		case CAM_FLASH_OFF:
			return CAM_MENU_FLASH_OFF;
		case CAM_FLASH_ON:
			return CAM_MENU_FLASH_ON;
		case CAM_FLASH_AUTO:
			return CAM_MENU_FLASH_AUTO;
		default:
			return CAM_MENU_FLASH_OFF;
	}
}

int cam_convert_recording_mode_to_menu_item(int mode)
{
	switch(mode)
	{
		case CAM_RECORD_NORMAL:
			return CAM_MENU_RECORDING_MODE_NORMAL;
		case CAM_RECORD_SELF:
			return CAM_MENU_RECORDING_MODE_SELF_NORMAL;
		case CAM_RECORD_MMS:
			return CAM_MENU_RECORDING_MODE_LIMIT;
		case CAM_RECORD_SELF_MMS:
			return CAM_MENU_RECORDING_MODE_SELF_LIMIT;
		case CAM_RECORD_SLOW:
			return CAM_MENU_RECORDING_MODE_SLOW;
		case CAM_RECORD_FAST:
			return CAM_MENU_RECORDING_MODE_FAST;
		default:
			return CAM_MENU_RECORDING_MODE_NORMAL;
	}
}

int cam_convert_video_resolution_to_menu_item(int mode)
{
	switch(mode)
	{

		case CAM_RESOLUTION_FULLHD:
			return CAM_MENU_VIDEO_RESOLUTION_FULLHD;
		case CAM_RESOLUTION_HD:
			return CAM_MENU_VIDEO_RESOLUTION_HD;
		case CAM_RESOLUTION_WVGA2:
			return CAM_MENU_VIDEO_RESOLUTION_WVGA;
		case CAM_RESOLUTION_VGA:
			return CAM_MENU_VIDEO_RESOLUTION_VGA;
		case CAM_RESOLUTION_QVGA:
			return CAM_MENU_VIDEO_RESOLUTION_QVGA;
		case CAM_RESOLUTION_QCIF:
			return CAM_MENU_VIDEO_RESOLUTION_QCIF;
		default:
			return CAM_MENU_VIDEO_RESOLUTION_QCIF;
	}
}

int cam_convert_photo_resolution_to_menu_item(int mode)
{
	switch(mode)
	{
		case CAM_RESOLUTION_3264x2448:
			return CAM_MENU_PHOTO_RESOLUTION_3264x2448;
		case CAM_RESOLUTION_3264x1960:
			return CAM_MENU_PHOTO_RESOLUTION_3264x1960;
		case CAM_RESOLUTION_3264x1836:
			return CAM_MENU_PHOTO_RESOLUTION_3264x1836;
		case CAM_RESOLUTION_2800x1920:
			return CAM_MENU_PHOTO_RESOLUTION_2800x1920;
		case CAM_RESOLUTION_2560x1536:
			return CAM_MENU_PHOTO_RESOLUTION_2560x1536;
		case CAM_RESOLUTION_2560x1440:
			return CAM_MENU_PHOTO_RESOLUTION_2560x1440;
		case CAM_RESOLUTION_2048x1536:
			return CAM_MENU_PHOTO_RESOLUTION_2048x1536;
		case CAM_RESOLUTION_2048x1152:
			return CAM_MENU_PHOTO_RESOLUTION_2048x1152;
		case CAM_RESOLUTION_1920x1080:
			return CAM_MENU_PHOTO_RESOLUTION_1920x1080;
		case CAM_RESOLUTION_1600x1200:
			return CAM_MENU_PHOTO_RESOLUTION_1600x1200;
		case CAM_RESOLUTION_1392x1392:
			return CAM_MENU_PHOTO_RESOLUTION_1392x1392;
		case CAM_RESOLUTION_1280x960:
			return CAM_MENU_PHOTO_RESOLUTION_1280x960;
		case CAM_RESOLUTION_1280x720:
			return CAM_MENU_PHOTO_RESOLUTION_1280x720;
		case CAM_RESOLUTION_VGA:
			return CAM_MENU_PHOTO_RESOLUTION_640x480;
		default:
			return CAM_MENU_PHOTO_RESOLUTION_640x480;
	}
}

int cam_convert_iso_to_menu_item(int mode)
{
	switch(mode)
	{
		case CAM_ISO_AUTO:
			return CAM_MENU_ISO_AUTO;
		case CAM_ISO_50:
			return CAM_MENU_ISO_50;
		case CAM_ISO_100:
			return CAM_MENU_ISO_100;
		case CAM_ISO_200:
			return CAM_MENU_ISO_200;
		case CAM_ISO_400:
			return CAM_MENU_ISO_400;
		case CAM_ISO_800:
			return CAM_MENU_ISO_80O;
		case CAM_ISO_1600:
			return CAM_MENU_ISO_1600;
		case CAM_ISO_3200:
			return CAM_MENU_ISO_320O;
		default:
			return CAM_MENU_ISO_AUTO;
	}
}

int cam_convert_focus_to_menu_item(int mode)
{
	switch(mode)
	{
		case CAM_FOCUS_AUTO:
			return CAM_MENU_FOCUS_AUTO;
		case CAM_FOCUS_MACRO:
			return CAM_MENU_FOCUS_MACRO;
		case CAM_FOCUS_FACE:
			return CAM_MENU_FOCUS_FACE_DETECTION;
		default:
			return CAM_MENU_FOCUS_AUTO;
	}
}

int cam_convert_timer_to_menu_item(int mode)
{
	switch(mode)
	{
		case CAM_SETTINGS_TIMER_OFF:
			return CAM_MENU_TIMER_OFF;
		case CAM_SETTINGS_TIMER_2SEC:
			return CAM_MENU_TIMER_2SEC;
		case CAM_SETTINGS_TIMER_5SEC:
			return CAM_MENU_TIMER_5SEC;
		case CAM_SETTINGS_TIMER_10SEC:
			return CAM_MENU_TIMER_10SEC;
		default:
			return CAM_MENU_TIMER_OFF;
	}
}

int cam_convert_scene_mode_to_menu_item(int mode)
{
	switch(mode)
	{
		case CAM_SCENE_NONE:
			return CAM_MENU_SCENE_AUTO;
		case CAM_SCENE_PORTRAIT:
			return CAM_MENU_SCENE_PORTRAIT;
		case CAM_SCENE_LANDSCAPE:
			return CAM_MENU_SCENE_LANDSCAPE;
		case CAM_SCENE_NIGHT:
			return CAM_MENU_SCENE_NIGHT;
		case CAM_SCENE_SPORTS:
			return CAM_MENU_SCENE_SPORTS;
		case CAM_SCENE_PARTY:
			return CAM_MENU_SCENE_PARTY;
		case CAM_SCENE_BEACHSNOW:
			return CAM_MENU_SCENE_BEACHSNOW;
		case CAM_SCENE_SUNSET:
			return CAM_MENU_SCENE_SUNSET;
		case CAM_SCENE_DUSKDAWN:
			return CAM_MENU_SCENE_DUSKDAWN;
		case CAM_SCENE_FALL:
			return CAM_MENU_SCENE_FALL;
		case CAM_SCENE_FIREWORK:
			return CAM_MENU_SCENE_FIREWORK;
		case CAM_SCENE_TEXT:
			return CAM_MENU_SCENE_TEXT;
		case CAM_SCENE_CANDLELIGHT:
			return CAM_MENU_SCENE_CANDLELIGHT;
		case CAM_SCENE_BACKLIGHT:
			return CAM_MENU_SCENE_BACKLIGHT;
		default:
			return CAM_MENU_SCENE_AUTO;
	}
}

int cam_convert_wb_to_menu_item(int mode)
{
	switch(mode)
	{
		case CAM_SETTINGS_WB_AWB:
			return CAM_MENU_WB_AWB;
		case CAM_SETTINGS_WB_INCANDESCENT:
			return CAM_MENU_WB_INCANDESCENT;
		case CAM_SETTINGS_WB_FLUORESCENT:
			return CAM_MENU_WB_FLUORESCENT;
		case CAM_SETTINGS_WB_DAYLIGHT:
			return CAM_MENU_WB_DAYLIGHT;
		case CAM_SETTINGS_WB_CLOUDY:
			return CAM_MENU_WB_CLOUDY;
		case CAM_SETTINGS_WB_SHADE:
			return CAM_MENU_WB_SHADE;
		case CAM_SETTINGS_WB_HORIZON:
			return CAM_MENU_WB_HORIZON;
		case CAM_SETTINGS_WB_FLASH:
			return CAM_MENU_WB_FLASH;
		case CAM_SETTINGS_WB_CUSTOM:
			return CAM_MENU_WB_CLOUDY;
		default:
			return CAM_MENU_WB_AWB;
	}
}

int cam_convert_effect_to_menu_item(int mode)
{
	switch(mode)
	{
		case CAM_SETTINGS_EFFECTS_NOR:
			return CAM_MENU_EFFECT_NONE;
		case CAM_SETTINGS_EFFECTS_NEGATIVE:
			return CAM_MENU_EFFECT_NEGATIVE;
		case CAM_SETTINGS_EFFECTS_MONO:
			return CAM_MENU_EFFECT_BLACKANDWHITE;
		case CAM_SETTINGS_EFFECTS_SEPIA:
			return CAM_MENU_EFFECT_SEPIA;
		default:
			return CAM_MENU_EFFECT_NONE;
	}
}

int cam_convert_metering_to_menu_item(int mode)
{
	switch(mode)
	{
		case CAM_METERING_MATRIX:
			return CAM_MENU_METERING_MATRIX;
		case CAM_METERING_CENTER_WEIGHTED:
			return CAM_MENU_METERING_CENTER_WEIGHTED;
		case CAM_METERING_SPOT:
			return CAM_MENU_METERING_SPOT;
		default:
			return CAM_MENU_METERING_MATRIX;
	}
}

int cam_convert_storage_to_menu_item(int mode)
{
	switch(mode)
	{
		case CAM_STORAGE_INTERNAL:
			return CAM_MENU_STORAGE_PHONE;
		case CAM_STORAGE_EXTERNAL:
			return CAM_MENU_STORAGE_MMC;
		default:
			return CAM_MENU_STORAGE_PHONE;
	}
}

int cam_convert_volume_key_to_menu_item(int mode)
{
	switch(mode)
	{
		case CAM_VOLUME_KEY_ZOOM:
			return CAM_MENU_VOLUME_KEY_ZOOM;
		case CAM_VOLUME_KEY_SHUTTER:
			return CAM_MENU_VOLUME_KEY_SHUTTER;
		default:
			return CAM_MENU_VOLUME_KEY_ZOOM;
	}
}

int cam_convert_menu_item_to_mmfw_value(int mode)
{
	switch(mode)
	{
		case CAM_MENU_SCENE_AUTO:
			return 	CAM_SCENE_NONE;
		case CAM_MENU_SCENE_PORTRAIT:
			return 	CAM_SCENE_PORTRAIT;
		case CAM_MENU_SCENE_LANDSCAPE:
			return 	CAM_SCENE_LANDSCAPE;
		case CAM_MENU_SCENE_NIGHT:
			return CAM_SCENE_NIGHT;
		case CAM_MENU_SCENE_SPORTS:
			return CAM_SCENE_SPORTS;
		case CAM_MENU_SCENE_PARTY:
			return CAM_SCENE_PARTY;
	 	case CAM_MENU_SCENE_BEACHSNOW:
			return CAM_SCENE_BEACHSNOW;
	 	case CAM_MENU_SCENE_SUNSET:
			return CAM_SCENE_SUNSET;
	 	case CAM_MENU_SCENE_DUSKDAWN:
			return CAM_SCENE_DUSKDAWN;
	 	case CAM_MENU_SCENE_FALL:
			return CAM_SCENE_FALL;
	 	case CAM_MENU_SCENE_FIREWORK:
			return CAM_SCENE_FIREWORK;
	 	case CAM_MENU_SCENE_TEXT:
			return CAM_SCENE_TEXT;
	 	case CAM_MENU_SCENE_CANDLELIGHT:
			return CAM_SCENE_CANDLELIGHT;
	 	case CAM_MENU_SCENE_BACKLIGHT:
			return CAM_SCENE_BACKLIGHT;
		case CAM_MENU_SHOOTING_SINGLE:
			return CAM_SINGLE_MODE;
		case CAM_MENU_SHOOTING_SELF_SINGLE:
			return CAM_SELF_SINGLE_MODE;
		case CAM_MENU_PHOTO_RESOLUTION_3264x2448:
			return CAM_RESOLUTION_3264x2448;
		case CAM_MENU_PHOTO_RESOLUTION_3264x1960:
			return CAM_RESOLUTION_3264x1960;
		case CAM_MENU_PHOTO_RESOLUTION_3264x1836:
			return CAM_RESOLUTION_3264x1836;
		case CAM_MENU_PHOTO_RESOLUTION_2800x1920:
			return CAM_RESOLUTION_2800x1920;
		case CAM_MENU_PHOTO_RESOLUTION_2560x1920:
			return CAM_RESOLUTION_2560x1920;
		case CAM_MENU_PHOTO_RESOLUTION_2560x1536:
			return CAM_RESOLUTION_2560x1536;
		case CAM_MENU_PHOTO_RESOLUTION_2560x1440:
			return CAM_RESOLUTION_2560x1440;
		case CAM_MENU_PHOTO_RESOLUTION_2048x1536:
			return CAM_RESOLUTION_2048x1536;
		case CAM_MENU_PHOTO_RESOLUTION_2048x1152:
			return CAM_RESOLUTION_2048x1152;
		case CAM_MENU_PHOTO_RESOLUTION_1920x1080:
			return CAM_RESOLUTION_1920x1080;
		case CAM_MENU_PHOTO_RESOLUTION_1600x1200:
			return CAM_RESOLUTION_1600x1200;
		case CAM_MENU_PHOTO_RESOLUTION_1392x1392:
			return CAM_RESOLUTION_1392x1392;
		case CAM_MENU_PHOTO_RESOLUTION_1280x960:
			return CAM_RESOLUTION_1280x960;
		case CAM_MENU_PHOTO_RESOLUTION_1280x720:
			return CAM_RESOLUTION_1280x720;
		case CAM_MENU_PHOTO_RESOLUTION_640x480:
			return CAM_RESOLUTION_VGA;
		case CAM_MENU_VIDEO_RESOLUTION_FULLHD:
			return CAM_RESOLUTION_FULLHD;
		case CAM_MENU_VIDEO_RESOLUTION_HD:
			return CAM_RESOLUTION_HD;
		case CAM_MENU_VIDEO_RESOLUTION_WVGA:
			return CAM_RESOLUTION_WVGA2;
		case CAM_MENU_VIDEO_RESOLUTION_VGA:
			return CAM_RESOLUTION_VGA;
		case CAM_MENU_VIDEO_RESOLUTION_QVGA:
			return CAM_RESOLUTION_QVGA;
		case CAM_MENU_VIDEO_RESOLUTION_QCIF:
			return CAM_RESOLUTION_QCIF;
		case CAM_MENU_WB_AWB:
			return CAM_SETTINGS_WB_AWB;
		case CAM_MENU_WB_INCANDESCENT:
			return CAM_SETTINGS_WB_INCANDESCENT;
		case CAM_MENU_WB_FLUORESCENT:
			return CAM_SETTINGS_WB_FLUORESCENT;
		case CAM_MENU_WB_DAYLIGHT:
			return CAM_SETTINGS_WB_DAYLIGHT;
		case CAM_MENU_WB_CLOUDY:
			return CAM_SETTINGS_WB_CLOUDY;
		case CAM_MENU_WB_SHADE:
			return CAM_SETTINGS_WB_SHADE;
		case CAM_MENU_WB_HORIZON:
			return CAM_SETTINGS_WB_HORIZON;
		case CAM_MENU_WB_FLASH:
			return CAM_SETTINGS_WB_FLASH;
		case CAM_MENU_WB_CUSTOM:
			return CAM_SETTINGS_WB_CUSTOM;
		case CAM_MENU_EFFECT_NONE:
			return CAM_SETTINGS_EFFECTS_NOR;
		case CAM_MENU_EFFECT_NEGATIVE:
			return CAM_SETTINGS_EFFECTS_NEGATIVE;
		case CAM_MENU_EFFECT_BLACKANDWHITE:
			return CAM_SETTINGS_EFFECTS_GRAY;
		case CAM_MENU_EFFECT_SEPIA:
			return CAM_SETTINGS_EFFECTS_SEPIA;
		case CAM_MENU_FOCUS_AUTO:
			return CAM_FOCUS_AUTO;
		case CAM_MENU_FOCUS_MACRO:
			return CAM_FOCUS_MACRO;
		case CAM_MENU_FOCUS_FACE_DETECTION:
			return CAM_FOCUS_FACE;
		case CAM_MENU_RECORDING_MODE_NORMAL:
			return CAM_RECORD_NORMAL;
		case CAM_MENU_RECORDING_MODE_LIMIT:
			return CAM_RECORD_MMS;
		case CAM_MENU_RECORDING_MODE_SLOW:
			return CAM_RECORD_SLOW;
		case CAM_MENU_RECORDING_MODE_FAST:
			return CAM_RECORD_FAST;
		case CAM_MENU_RECORDING_MODE_SELF_NORMAL:
			return CAM_RECORD_SELF;
		case CAM_MENU_RECORDING_MODE_SELF_LIMIT:
			return CAM_RECORD_SELF_MMS;
 		case CAM_MENU_FLASH_OFF:
			return CAM_FLASH_OFF;
		case CAM_MENU_FLASH_ON:
			return CAM_FLASH_ON;
		case CAM_MENU_FLASH_AUTO:
			return CAM_FLASH_AUTO;
		case CAM_MENU_METERING_MATRIX:
			return CAM_METERING_MATRIX;
		case CAM_MENU_METERING_CENTER_WEIGHTED:
			return CAM_METERING_CENTER_WEIGHTED;
		case CAM_MENU_METERING_SPOT:
			return CAM_METERING_SPOT;
		case CAM_MENU_ISO_AUTO:
			return CAM_ISO_AUTO;
		case CAM_MENU_ISO_50:
			return CAM_ISO_50;
		case CAM_MENU_ISO_100:
			return CAM_ISO_100;
		case CAM_MENU_ISO_200:
			return CAM_ISO_200;
		case CAM_MENU_ISO_400:
			return CAM_ISO_400;
		case CAM_MENU_ISO_80O:
			return CAM_ISO_800;
		case CAM_MENU_ISO_1600:
			return CAM_ISO_1600;
		case CAM_MENU_ISO_320O:
			return CAM_ISO_3200;

		default:
			return -1;

	}
}

int cam_get_menu_item_to_index(void* data, int menu_type, int item)
{
	cam_menu_composer* menu_composer = NULL;
	menu_composer = calloc(1, sizeof(cam_menu_composer));

	switch((CAM_MENU_ITEM)menu_type)
	{
		case CAM_MENU_SHOOTING_MODE:
			cam_compose_menu_shooting_mode(data, menu_composer);
			break;

		case CAM_MENU_FLASH:
			cam_compose_menu_flash(data, menu_composer);
			break;

		case CAM_MENU_RECORDING_MODE:
			cam_compose_menu_recording_mode(data, menu_composer);
			break;

		case CAM_MENU_PHOTO_RESOLUTION:
			cam_compose_menu_photo_resolution(data, menu_composer);
			break;

		case CAM_MENU_VIDEO_RESOLUTION:
			cam_compose_menu_video_resolution(data, menu_composer);
			break;

		case CAM_MENU_ISO:
			cam_compose_menu_iso(data, menu_composer);
			break;

		case CAM_MENU_FOCUS_MODE:
			cam_compose_menu_focus_mode(data, menu_composer);
			break;

		case CAM_MENU_SCENE_MODE:
			cam_compose_menu_scene_mode(data, menu_composer);
			break;

		case CAM_MENU_WHITE_BALANCE:
			cam_compose_menu_wb(data, menu_composer);
			break;

		case CAM_MENU_EFFECTS:
			cam_compose_menu_effect(data, menu_composer);
			break;

		case CAM_MENU_METERING:
			cam_compose_menu_metering(data, menu_composer);
			break;

		case CAM_MENU_STORAGE:
			cam_compose_menu_storage(data, menu_composer);
			break;

		case CAM_MENU_VOLUME_KEY:
			cam_compose_menu_volume_key(data, menu_composer);
			break;

		default:
			break;
	}

	CAM_MENU_ITEM* menu_item  = NULL;
	int size = eina_array_count(menu_composer->array);


	int index = 0;
	for (index = 0; index < size; index++) {
		menu_item = (CAM_MENU_ITEM*)eina_array_data_get(menu_composer->array, index);

		if((CAM_MENU_ITEM)item == *menu_item) {
			cam_compose_free(menu_composer);
			return index;
		}
	}

	cam_compose_free(menu_composer);
	return INVALID_INDEX;
}


int cam_get_value_to_index(void* data, int menu_type, int value)
{
	cam_menu_composer* menu_composer = NULL;
	menu_composer = calloc(1, sizeof(cam_menu_composer));

	int item = 0;

	switch(menu_type)
	{
		case CAM_MENU_SHOOTING_MODE:
			cam_compose_menu_shooting_mode(data, menu_composer);
			item = cam_convert_shot_mode_to_menu_item(value);
			break;

		case CAM_MENU_FLASH:
			cam_compose_menu_flash(data, menu_composer);
			item = cam_convert_flash_to_menu_item(value);
			break;

		case CAM_MENU_RECORDING_MODE:
			cam_compose_menu_recording_mode(data, menu_composer);
			item = cam_convert_recording_mode_to_menu_item(value);
			break;

		case CAM_MENU_PHOTO_RESOLUTION:
			cam_compose_menu_photo_resolution(data, menu_composer);
			item = cam_convert_photo_resolution_to_menu_item(value);
			break;

		case CAM_MENU_VIDEO_RESOLUTION:
			cam_compose_menu_video_resolution(data, menu_composer);
			item = cam_convert_video_resolution_to_menu_item(value);
			break;

		case CAM_MENU_ISO:
			cam_compose_menu_iso(data, menu_composer);
			item = cam_convert_iso_to_menu_item(value);
			break;

		case CAM_MENU_FOCUS_MODE:
			cam_compose_menu_focus_mode(data, menu_composer);
			item = cam_convert_focus_to_menu_item(value);
			break;

		case CAM_MENU_TIMER:
			cam_compose_menu_timer_mode(data, menu_composer);
			item = cam_convert_timer_to_menu_item(value);
			break;

		case CAM_MENU_SCENE_MODE:
			cam_compose_menu_scene_mode(data, menu_composer);
			item = cam_convert_scene_mode_to_menu_item(value);
			break;

		case CAM_MENU_WHITE_BALANCE:
			cam_compose_menu_wb(data, menu_composer);
			item = cam_convert_wb_to_menu_item(value);
			break;

		case CAM_MENU_EFFECTS:
			cam_compose_menu_effect(data, menu_composer);
			item = cam_convert_effect_to_menu_item(value);
			break;

		case CAM_MENU_METERING:
			cam_compose_menu_metering(data, menu_composer);
			item = cam_convert_metering_to_menu_item(value);
			break;

		case CAM_MENU_STORAGE:
			cam_compose_menu_storage(data, menu_composer);
			item = cam_convert_storage_to_menu_item(value);
			break;

		case CAM_MENU_VOLUME_KEY:
			cam_compose_menu_volume_key(data, menu_composer);
			item = cam_convert_volume_key_to_menu_item(value);
			break;

		default:
			break;
	}

	CAM_MENU_ITEM* menu_item  = NULL;
	int size = eina_array_count(menu_composer->array);

	int index = 0;
	for (index = 0; index < size; index++) {
		menu_item = (CAM_MENU_ITEM*)eina_array_data_get(menu_composer->array, index);

		if((CAM_MENU_ITEM)item == *menu_item) {
			cam_compose_free(menu_composer);
			return index;
		}
	}

	cam_compose_free(menu_composer);
	return INVALID_INDEX;
}

int cam_get_sound_index(int sound)
{
	return sound + 1;
}

int shot_mode_set(void *data, int mode)
{
	CAM_UI_LOG("%d", mode);
	struct appdata *ad = (struct appdata *)data;
	CamAppData *camapp = NULL;
	cam_retvm_if(ad == NULL, -1, "appdata is NULL");
	camapp = ad->camapp_handle;
	cam_retvm_if(camapp == NULL, -1, "camapp_handle is NULL");

	GValue value = { 0 };
	if (mode == CAM_SINGLE_MODE)
		camapp->enable_touch_af = TRUE;
	else
		camapp->enable_touch_af = FALSE;

	CAM_GVALUE_SET_INT(value, mode);
	if (mode != camapp->shooting_mode) {
		cam_handle_value_set(ad, PROP_SHOT_MODE, &value);
	}
	return 0;
}



