/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#include "cam.h"
#include "cam_zoom_edc_callback.h"
#include "cam_capacity_type.h"
#include "cam_device_capacity.h"


#define ZOOM_DESTROY_TIME	3.0


static Evas_Object *slider = NULL;
static Ecore_Timer *zoom_destroy_timer = NULL;
static int zoom_value[CAM_ZOOM_VALUE_MAX] = { 0 };

static gboolean __init_zoom_value(struct appdata *ad);
static Eina_Bool __zoom_destroy_timer_cb(void *data);
static void __set_zoom_value(struct appdata *data, int zoom_level);
static void __zoom_slider_stop_cb(void *data, Evas_Object *obj, void *event_info);
static void __zoom_slider_changed_cb(void *data, Evas_Object *obj, void *event_info);
static void __set_gauge_focus(struct appdata *data);


static gboolean __init_zoom_value(struct appdata *ad)
{
	cam_retvm_if(ad == NULL, FALSE, "appdata is NULL");
	CamAppData *camapp = ad->camapp_handle;
	cam_retvm_if(camapp == NULL, FALSE, "camapp_handle is NULL");

	int i = 0;
	double gap = (double)(camapp->zoom_max - camapp->zoom_min) / (CAM_ZOOM_VALUE_MAX - 2);
	for (i = 2; i < CAM_ZOOM_VALUE_MAX - 1; i++) {
		zoom_value[i] = (int)(0.5 + camapp->zoom_min + (i - 1) * gap);
		DEBUG_TRACE("zoom_value[%d] =%d", i, zoom_value[i]);
	}
	zoom_value[1] = camapp->zoom_min;
	DEBUG_TRACE("zoom_value[%d] =%d", 1, zoom_value[1]);
	zoom_value[CAM_ZOOM_VALUE_MAX - 1] = camapp->zoom_max;
	DEBUG_TRACE("zoom_value[%d] =%d", (CAM_ZOOM_VALUE_MAX - 1), zoom_value[CAM_ZOOM_VALUE_MAX - 1]);

	return TRUE;
}

static Eina_Bool __zoom_destroy_timer_cb(void *data)
{
	CAM_UI_LOG();

	struct appdata *ad = (struct appdata *)data;
	cam_retvm_if(ad == NULL, ECORE_CALLBACK_CANCEL, "appdata is NULL");

	unload_zoom_edje(ad);

	if (zoom_destroy_timer)
		zoom_destroy_timer = NULL;

	return ECORE_CALLBACK_CANCEL;
}

static void __set_zoom_value(struct appdata *data, int zoom_level)
{
	struct appdata *ad = (struct appdata *)data;
	cam_retm_if(ad == NULL, "appdata is NULL");
	CamAppData *camapp = ad->camapp_handle;
	cam_retm_if(camapp == NULL, "camapp_handle is NULL");

	int zoom = 0;
	GValue value = { 0, };

	camapp->zoom_mode = zoom_level;

	zoom = zoom_value[camapp->zoom_mode];
	DEBUG_TRACE("zoom_value =%d", zoom);
	CAM_GVALUE_SET_INT(value, zoom);

	if (!cam_handle_value_set(ad, PROP_ZOOM, &value)) {
		cam_critical(LOG_CAM, "cam_handle_value_set failed");
	}
}

static void __zoom_slider_stop_cb(void *data, Evas_Object *obj, void *event_info)
{
	struct appdata *ad = (struct appdata *)data;
	cam_retm_if(ad == NULL, "appdata is NULL");

	if (obj) {
		DEBUG_TRACE("");
		Evas_Object *slider = (Evas_Object *)obj;
		int slider_value = (int)elm_slider_value_get(slider);
		elm_slider_value_set(slider, slider_value);
		__set_zoom_value(ad, slider_value);
	} else {
		cam_critical(LOG_UI, "Error : slider is NULL");
	}

	if(zoom_destroy_timer)
		ecore_timer_reset(zoom_destroy_timer);
}

static void __zoom_slider_changed_cb(void *data, Evas_Object *obj, void *event_info)
{
	struct appdata *ad = (struct appdata *)data;
	cam_retm_if(ad == NULL, "appdata is NULL");

	if (obj) {
		Evas_Object *slider = (Evas_Object *)obj;
		int slider_value = (int)elm_slider_value_get(slider);
		DEBUG_TRACE("slider_value:%d", slider_value);
		__set_zoom_value(ad, slider_value);
	} else {
		cam_critical(LOG_UI, "Error : slider is NULL");
	}

	if(zoom_destroy_timer)
		ecore_timer_reset(zoom_destroy_timer);
}

gboolean load_zoom_edje(struct appdata *ad)
{
	DEBUG_TRACE("START");

	cam_retvm_if(ad == NULL, FALSE, "appdata is NULL");
	CamAppData *camapp = ad->camapp_handle;
	cam_retvm_if(camapp == NULL, FALSE, "camapp_handle is NULL");

	elm_object_part_content_unset(ad->ug_base, "zoom_layout");
	if (ad->zoom_edje)
		cam_zoom_unload_edje(ad);

	/* zoom bg */
	gboolean portrait = FALSE;
	gboolean slider_inverse = FALSE;
	switch (ad->target_direction) {
	case CAM_TARGET_DIRECTION_LANDSCAPE:
		ad->zoom_edje = cam_app_load_edj(ad->ug_base, CAM_ZOOM_EDJ_NAME, GRP_MAIN);
		portrait = FALSE;
		slider_inverse = TRUE;
		break;
	case CAM_TARGET_DIRECTION_LANDSCAPE_INVERSE:
		ad->zoom_edje = cam_app_load_edj(ad->ug_base, CAM_ZOOM_EDJ_INVERSE_NAME, GRP_MAIN);
		portrait = FALSE;
		slider_inverse = FALSE;
		break;
	case CAM_TARGET_DIRECTION_PORTRAIT:
		ad->zoom_edje = cam_app_load_edj(ad->ug_base, CAM_ZOOM_EDJ_VERTICAL_NAME, GRP_MAIN);
		portrait = TRUE;
		slider_inverse = FALSE;
		break;
	case CAM_TARGET_DIRECTION_PORTRAIT_INVERSE:
		ad->zoom_edje = cam_app_load_edj(ad->ug_base, CAM_ZOOM_EDJ_VERTICAL_INVERSE_NAME, GRP_MAIN);
		slider_inverse = TRUE;
		portrait = TRUE;
		break;
	default:
		DEBUG_TRACE("wow, reached un-able reached codes");
	}

	cam_retvm_if(ad->zoom_edje == NULL, FALSE, "zom_edje is NULL");
	elm_object_part_content_set(ad->ug_base, "zoom_layout", ad->zoom_edje);

	/* zoom text */
	if (!ad->zoom_text_min_edje) {
		ad->zoom_text_min_edje = (Evas_Object *)edje_object_part_object_get(_EDJ(ad->zoom_edje), "text_min");
	}
	cam_retvm_if(ad->zoom_text_min_edje == NULL, FALSE, "zoom_text_min is NULL");

	if (!ad->zoom_text_max_edje) {
		ad->zoom_text_max_edje = (Evas_Object *)edje_object_part_object_get(_EDJ(ad->zoom_edje), "text_max");
	}
	cam_retvm_if(ad->zoom_text_max_edje == NULL, FALSE, "zoom_text_max is NULL");

	edje_object_part_text_set(_EDJ(ad->zoom_edje), "text_min", "x1");
	edje_object_part_text_set(_EDJ(ad->zoom_edje), "text_max", "x4");

	/* zoom slider */
	slider = elm_slider_add(ad->zoom_edje);
	cam_retvm_if(slider == NULL, FALSE, "slider is NULL");

	elm_object_style_set(slider, "camera");
	if (portrait) {
		elm_slider_horizontal_set(slider, EINA_TRUE);
	} else {
		elm_slider_horizontal_set(slider, EINA_FALSE);
	}
	if (slider_inverse) {
		elm_slider_inverted_set(slider, EINA_TRUE);
	} else {
		elm_slider_inverted_set(slider, EINA_FALSE);
	}
	elm_slider_min_max_set(slider, CAM_ZOOM_VALUE_MIN + 1, CAM_ZOOM_VALUE_MAX - 1);


	elm_slider_value_set(slider, camapp->zoom_mode);

	int slider_value = (int)elm_slider_value_get(slider);
	DEBUG_TRACE("slider_value:%d / camapp->zoom_mode:%d", slider_value, camapp->zoom_mode);

	evas_object_size_hint_align_set(slider, EVAS_HINT_FILL, EVAS_HINT_FILL);
	evas_object_size_hint_weight_set(slider, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_object_part_content_set(ad->zoom_edje, "slider", slider);

	evas_object_smart_callback_del(slider, "slider,drag,stop", __zoom_slider_stop_cb);
	evas_object_smart_callback_add(slider, "slider,drag,stop", __zoom_slider_stop_cb, (void*)ad);
	evas_object_smart_callback_del(slider, "changed", __zoom_slider_changed_cb);
	evas_object_smart_callback_add(slider, "changed", __zoom_slider_changed_cb, (void*)ad);

	return TRUE;;
}

gboolean update_zoom_edje(struct appdata *ad)
{
	DEBUG_TRACE("START");

	cam_retvm_if(ad == NULL, FALSE, "appdata is NULL");
	CamAppData *camapp = ad->camapp_handle;
	cam_retvm_if(camapp == NULL, FALSE, "camapp_handle is NULL");

	cam_retvm_if(ad->zoom_edje == NULL, FALSE, "zom_edje is NULL");
	cam_retvm_if(slider == NULL, FALSE, "zoom slider is NULL");

	elm_slider_value_set(slider, camapp->zoom_mode);

	return TRUE;
}

gboolean unload_zoom_edje(struct appdata *ad)
{
	cam_retvm_if(ad == NULL, FALSE, "appdata is NULL");

	cam_reset_focus_mode(ad);

	DEL_EVAS_OBJECT(ad->zoom_text_min_edje);
	DEL_EVAS_OBJECT(ad->zoom_text_max_edje);
	DEL_EVAS_OBJECT(ad->zoom_edje);
	REMOVE_TIMER(zoom_destroy_timer);

	return TRUE;
}

gboolean zoom_in(struct appdata *ad, gboolean is_zoom_in, int zoom_gap)
{
	CamAppData *camapp = NULL;
	cam_retvm_if(ad == NULL, FALSE, "appdata is NULL");
	camapp = ad->camapp_handle;
	cam_retvm_if(camapp == NULL, FALSE, "camapp_handle is NULL");

	int zoom_value = 0;
	unsigned int capacity = 0;

	GetCamDevFuncCaps(&capacity, ad);
	if (!(capacity & CAM_CP_FUNC_ZOOM)) {
		DEBUG_TRACE("Zoom is not supported");
		return FALSE;
	}

	REMOVE_TIMER(zoom_destroy_timer);

	__init_zoom_value(ad);

	/* set zoom value */
	zoom_value = camapp->zoom_mode;

	if (is_zoom_in) {
		if (zoom_value < CAM_ZOOM_VALUE_MAX - 1)
			zoom_value += zoom_gap;
	} else {
		if (zoom_value > CAM_ZOOM_VALUE_MIN + 1)
			zoom_value -= zoom_gap;
	}
	DEBUG_TRACE("zoom_value = %d", zoom_value);

	__set_zoom_value(ad, zoom_value);

	if (ad->zoom_edje == NULL)
		load_zoom_edje(ad);
	else
		update_zoom_edje(ad);

	zoom_destroy_timer = ecore_timer_add(ZOOM_DESTROY_TIME, __zoom_destroy_timer_cb, ad);

	return TRUE;
}

void cam_zoom_create_pinch_edje(struct appdata *data)
{
	struct appdata *ad = (struct appdata *)data;
	cam_retm_if(ad == NULL, "appdata is NULL");

	if (ad->pinch_edje) {
		evas_object_del(ad->pinch_edje);
		ad->pinch_edje = NULL;
	}

	elm_object_part_content_unset(ad->ug_base, "pinch_layout");

	switch (ad->target_direction) {
	case CAM_TARGET_DIRECTION_LANDSCAPE:
	case CAM_TARGET_DIRECTION_LANDSCAPE_INVERSE:
		ad->pinch_edje = cam_app_load_edj(ad->ug_base, CAM_ZOOM_EDJ_NAME, "pinch");
		break;
	case CAM_TARGET_DIRECTION_PORTRAIT:
	case CAM_TARGET_DIRECTION_PORTRAIT_INVERSE:
		ad->pinch_edje = cam_app_load_edj(ad->ug_base, CAM_ZOOM_EDJ_VERTICAL_NAME, "pinch");
		break;
	default:
		DEBUG_TRACE("error target direction!");
		break;
	}

	cam_retm_if(ad->pinch_edje == NULL, "pinch_edje is NULL");
	elm_object_part_content_set(ad->ug_base, "pinch_layout", ad->pinch_edje);

	__set_gauge_focus(ad);
	__init_zoom_value(ad);
}

static void __set_gauge_focus(struct appdata *data)
{
	struct appdata *ad = (struct appdata *)data;
	CamAppData *camapp = NULL;
	cam_retm_if(ad == NULL, "appdata is NULL");
	camapp = ad->camapp_handle;
	cam_retm_if(camapp == NULL, "camapp_handle is NULL");

	DEBUG_TRACE("camapp->zoom_mode %d", camapp->zoom_mode);

	char part_name[CAM_ZOOM_VALUE_MAX + 1][128] = {
		{"set.pinch.default"},
		{"set.pinch.default"},
		{"set.pinch.1.1x"},
		{"set.pinch.1.2x"},
		{"set.pinch.1.3x"},
		{"set.pinch.1.4x"},
		{"set.pinch.1.5x"},
		{"set.pinch.1.6x"},
		{"set.pinch.1.7x"},
		{"set.pinch.1.8x"},
		{"set.pinch.1.9x"},
		{"set.pinch.2.0x"},
		{"set.pinch.2.1x"},
		{"set.pinch.2.2x"},
		{"set.pinch.2.3x"},
		{"set.pinch.2.4x"},
		{"set.pinch.2.5x"},
		{"set.pinch.2.6x"},
		{"set.pinch.2.7x"},
		{"set.pinch.2.8x"},
		{"set.pinch.2.9x"},
		{"set.pinch.3.0x"},
		{"set.pinch.3.1x"},
		{"set.pinch.3.2x"},
		{"set.pinch.3.3x"},
		{"set.pinch.3.4x"},
		{"set.pinch.3.5x"},
		{"set.pinch.3.6x"},
		{"set.pinch.3.7x"},
		{"set.pinch.3.8x"},
		{"set.pinch.3.9x"},
		{"set.pinch.4.0x"},
		{"set.pinch.4.0x"},
	};

	if ((camapp->zoom_mode >= CAM_ZOOM_VALUE_MIN)
		&&(camapp->zoom_mode <= CAM_ZOOM_VALUE_MAX)) {
		elm_object_signal_emit(ad->pinch_edje, part_name[camapp->zoom_mode], "prog");
	} else {
		DEBUG_TRACE("NOTE: error zoom mode");
	}

	evas_object_show(ad->pinch_edje);
}

gboolean cam_pinch_zoom_in(struct appdata* ad,gboolean is_zoom_in,int zoom_value)
{
	CamAppData *camapp = NULL;
	cam_retvm_if(ad == NULL, FALSE, "appdata is NULL");
	camapp = ad->camapp_handle;
	cam_retvm_if(camapp == NULL, FALSE, "camapp_handle is NULL");

	unsigned int capacity = 0;

	GetCamDevFuncCaps(&capacity, ad);
	if (!(capacity & CAM_CP_FUNC_ZOOM)) {
		DEBUG_TRACE("Zoom is not supported");
		return FALSE;
	}

	if (is_zoom_in) {
		if (camapp->zoom_mode < CAM_ZOOM_VALUE_MAX - 1) {
			camapp->zoom_mode = camapp->zoom_mode + zoom_value;
		}
	} else {
		if (camapp->zoom_mode > CAM_ZOOM_VALUE_MIN + 1) {
			camapp->zoom_mode = camapp->zoom_mode - zoom_value;
		}
	}

	__set_gauge_focus(ad);
	__set_zoom_value(ad, camapp->zoom_mode);

	return TRUE;
}
//endfile

